#!/usr/bin/perl -w

use strict;

use XML::Twig;

die "usage: perl gutxml2xhtml.pl filename\n" unless scalar @ARGV == 1;

my ($file) = $ARGV[0];

use vars qw($chapter_count $para_count);

my $start_tag_handlers = {'chapter' => \&chapter_start,
                           gutbook           => sub { $_[1]->set_gi('html');},
                           acknowledge       => \&plain_paragraph
                         };

my $twig_handlers = {'chapter/para' => \&para_main,
                      book                   => \&shift_element,
                     'chapter/title'         => \&chapter_title,
                      titlepage              => \&titlepage,
                      frontmatter            => \&shift_element,
                      part                   => \&shift_element,
                      chapter                => \&shift_element,
                      bookbody               => \&bookbody,
                     'titlepage/para'        => \&plain_paragraph, 
                     'titlepage/author'      => \&plain_paragraph,
                     'titlepage/title'       => \&set_title
                    };

my $twig= new XML::Twig(LoadDTD          =>   1,
                        DTDHandler       =>   \&set_dtd,
                        TwigRoots        => { book => 1},
                        StartTagHandlers => $start_tag_handlers,
                        TwigHandlers     => $twig_handlers);



$twig->parsefile($file);    # build the twig

$twig->print;

sub set_dtd {
    my ($twig, $dtd) = @_;
    my $internal = qq|\nPUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"\n"DTD/xhtml1-strict.dtd"|;
    $twig->twig_doctype('html', undef, undef, $internal);
}

sub chapter_start {     
    my ($twig, $chapter) = @_;
    $chapter_count++;
    $para_count = '0';
}

sub chapter_title {
    my ($twig, $title) = @_;
    my $html_anchor = XML::Twig::Elt->new('a');
    my $anchor_name = $chapter_count . '.0';
    $html_anchor->set_att('name', $anchor_name);
    $html_anchor->set_empty;
    $html_anchor->paste('first_child', $title);
    $title->set_gi('h2');
}

sub para_main {
    my ($twig, $para) = @_;
    $para_count++;
    my $para_id = $chapter_count . '.' . $para_count;
    $para->set_gi('p');
    my $html_anchor = XML::Twig::Elt->new('a');
    $html_anchor->set_att('name', $para_id);
    $html_anchor->set_empty;  
    $html_anchor->paste('first_child', $para);
}

sub set_title {
    my ($twig, $title) = @_;
    my $html_title = $title->copy;
    my $html = $title->parent('html');
    my $head_element = XML::Twig::Elt->new('head');
    $html_title->paste($head_element);
    $head_element->paste('first_child', $html);
    $title->set_gi('h1');
}

sub titlepage {
    my ($twig, $element) = @_;
    $element->set_gi('div');
    $element->set_att('class', 'intro');
}

sub shift_element {
    my ($twig, $element) = @_;
    $element->erase;
}

sub plain_paragraph {
    my ($twig, $element) = @_;
    $element->set_gi('p');;
}

sub bookbody {
    my ($twig, $bookbody) = @_;
    my $html = $bookbody->parent('html');
    my $body_element = XML::Twig::Elt->new('body');
    my $title_page = $bookbody->sibling(-1);
    my $acknowledge = $bookbody->sibling(-2);
    $acknowledge->move('last_child', $body_element);
    $bookbody->move('last_child', $body_element);
    $title_page->move('first_child', $body_element);
    $body_element->paste('last_child', $html);
    $bookbody->erase;
}
