#!/usr/bin/perl -w

use strict;

use XML::Twig;
use HTML::Entities;

die "usage: perl xhtml_search.pl [word | \"multi-word phrase\"] [filename]\n" unless scalar @ARGV == 2;

my ($match_word, $file) = @ARGV;


my ($current_chapter, $last_chapter, $global_match);

#my $file = 'files/iliad10.xml';
my $twig= new XML::Twig(TwigHandlers => { 'p'  => \&para, 'h2' => \&chapter_title},
                        TwigRoots    => {p => 1, 'h2' => 1, body => 1});



$twig->parsefile($file);    # build the twig

$twig->print;

warn "Sorry, no matches found for '$match_word'\n" unless $global_match;

sub para {
    my ($twig, $para) = @_;
    my $para_text = $para->text;

    if ($para_text =~ /\b(.{0,30}\b$match_word.{0,30}\b)/is) {
        my $snippet = $1;
        $snippet =~ s/\n/ /g;
        $snippet = decode_entities($snippet);
        $global_match++;
        my $anchor = $para->first_child;
        my $para_ref = $anchor->att('name');
        my $link = XML::Twig::Elt->new('a');
        $link->set_att('href', $file . '#' . $para_ref);
        $link->set_text($para_ref);
        
        $para->set_text(" - ...$snippet...");
        $link->paste('first_child', $para);

        if ((!$last_chapter) || ($last_chapter ne $current_chapter)) {
            my $header = XML::Twig::Elt->new('h2');
            $header->set_text($current_chapter);
            $header->paste('first_child', $para);
        }

        $last_chapter = $current_chapter;
    }
    else {
        $para->delete;
    }
}

sub chapter_title {     
    my ($twig, $chapter_title) = @_;
    $current_chapter = $chapter_title->text;
    $chapter_title->delete;
}
